% Jason R. Blevins and Garrett T. Senney
% Dynamic Selection and Distributional Bounds on Search Costs in Dynamic Unit-Demand Models
% Quantitative Economics
%
% Estimation of the period-by-period search cost distributions

format long;
tic;

n = k;                                  % Number of firms

MU_All = zeros(NP,k);
D = zeros(NP,k);
R_All = zeros(NP,1);

if resample == 1
    datasubsamp = DCS_resample(prices, W2011, k);
else
    datasubsamp = prices;
end

% Convert prices to (negative) utilities
if W2011 == 1
    datasubsamp = DCS_utilities(datasubsamp);
end

% Pregenerate and store random starting values
nstart = 10;
for j = 1:nstart
    init(j,:) = rand(1, n);
    init(j,:) = init(j,:) / sum(init(j,:));
end

% Estimate the q_i's (including q_N) using the restriction that the
% q_i's should be between 0 and 1. Later divide by the sum of all
% q_i's to make sure the sum is equal to 1.
low=zeros(n,1);
up=ones(n,1);

% Optimization options
opts=optimset('DerivativeCheck','off','Display','off','TolX',1e-6,'TolFun',1e-6,'TolCon',1e-5,'TolPCG',1e-5,'MaxIter',1000,'MaxFunEvals',10000,'GradObj','on','Hessian','off','Algorithm','interior-point');

parfor t = 1:NP
    disp(sprintf('%s Estimation for Period t = %d...', method_name, t));

    pricemat = datasubsamp{t};
    pricemat = pricemat(:,2);                % Remove first column of type IDs
    pricemat = pricemat(isfinite(pricemat)); % Remove NaN's
    %m = length(pricemat);                   % number of observations
    %pricemat = sort(pricemat(:));
    if W2011 == 1
        pricemat = -1 * pricemat;           % price ~= negative utility
    end
    plow = min(pricemat);                   % lower bound
    v = max(pricemat);                      % upper bound

    % Remove price observations that are equal to either plow or v
    ind = (pricemat ~= plow) & (pricemat ~= v);
    p = pricemat(ind);

    % Use an anonymous function to avoid global variables
    loglik_func = @(vect)loglik(vect, n, v, p, plow);

    % Use random draws or optimum from previous run as starting value
    if (resample == 0)
        loglik_init = zeros(nstart,1);
        for j = 1:nstart
            loglik_init(j) = loglik_func(init(j,:)');
        end
        [ loglik_min, j_min ] = min(loglik_init);
        init_best = init(j_min,:)';
    else
        init_best = vect_obs(t,:)';
    end

    [vect,lik,exit,OUTPUT,LAMBDA,GRAD,HES]=fmincon(loglik_func,init_best,[],[],[],[],low,up,[],opts);

    % Save optimum for later replications
    if (resample == 0)
        vect_obs(t,:) = vect;
    end

    qmat=vect(:,1)/sum(vect(:,1)); % q_i's have to sum up to 1

    MU_All(t,:) = qmat';

    % Calculate unit cost r
    ksum=0;
    for j=1:n;
        ksum=ksum+j*qmat(j);
    end
    r= (plow*ksum-v*qmat(1))/(ksum-qmat(1));

    R_All(t,1) = r;

    % Calculating Cutoff Search Cost
    i = 1;
    aa = 0;
    c = zeros(n,1);
    dc = zeros(n-1,n-1);
    for a = 1:n-1
        condc_func = @(z)condc(z, a, v, r, qmat, n);
        % Calculate critical search cost values C_i
        c(a,1) = quadl(condc_func,0,1,[],[]);
        for aa = 1:n-1
            conddc_func = @(z)conddc(z, a, aa, v, r, qmat, n, plow);
            % Calculate derivatives critical search cost values Delta_i with
            % respect to q_i's
            dc(a,aa) = quadl(conddc_func, 0, 1, [], []);
        end
    end

    D(t,:) = c';

    C = zeros(n-1,1);
    for z = 1:n-1;
        qsum = 0;
        for a = 1:z;
            qsum = qsum + qmat(a);
        end;
        C(z,1) = 1 - qsum; % Corresponding search cost CDF values
    end

end

C_All = sort(D,2);

telapsed = toc;
disp(sprintf('\nElapsed time: %g (s)', telapsed));
