% Jason R. Blevins and Garrett T. Senney
% Dynamic Selection and Distributional Bounds on Search Costs in Dynamic Unit-Demand Models
% Quantitative Economics
%
% Main script for estimation and bootstrap standard errors

% Default settings
if ~exist('book')
    book = 'Amemiya';                   % book = "Amemiya" or "Cameron"
end
if ~exist('k')
    k = 15;                             % k = 10, 15, 19
end
if ~exist('NP')
    NP = 236;                           % Number of periods (days) of data
end
if ~exist('REP')
    REP = 25;                           % Number of repetitions
end
if ~exist('monotonicity')
    monotonicity = 1;                   % Restrict to monotonic policies?
end
if ~exist('W2011')
    W2011 = 1;                          % Wildenbeest (2011) vertical differentiation
end

% Set the random number generator seed for reproducibility
rng(1117, 'twister');

% Load Price Data
load(sprintf('%s_Prices.mat', book));

% Start worker pool
ownpool = 0;
if (isempty(gcp('nocreate')))
    parpool();
    ownpool = 1;
end

% Set method name
if (W2011 == 0)
    method_name = 'MGW';
else
    method_name = 'W2011';
end

% Print summary statistics
DCS_SS(prices, DCS_utilities(prices), NP);

% Estimation Procedure for both MGW and W2011
resample = 0;
vect_obs = 1/k * ones(NP, k);
Bootstrapping_Estimation
G_c_All = DCS_MU_to_G(NP, k, MU_All);
MC_obs = R_All;
MC_mean_obs = mean(R_All);

% Save results
matfile = sprintf('%s_%s_Results_N=%d.mat', book, method_name, k);
save(matfile, '-regexp', '^(MC|MU|R|C|X|G_c)_All', 'k', 'NP');

Precalculate_H_Bounds
BOUNDS_min_ub_H_obs = min_ub_H;
BOUNDS_max_lb_H_obs = max_lb_H;
BOUNDS_avg_H_obs = avg_H;
BOUNDS_c_H_obs = c_H;

% Non-Linear Least Squares Parametric Estimation of H(c|\mu)
DCS_Master_NLS
NLLS_p_obs = p_values;
NLLS_theta_expo_obs = theta_expo;
NLLS_q_expo_obs = quantiles_expo;
NLLS_theta_mix_obs = theta_mix;
NLLS_q_mix_obs = quantiles_mix;

% Estimation of Quantiles of G
DCS_Report_G_Quantiles
QUANTILE_G_p_obs = p_values;
QUANTILE_G_q_obs = quantiles';
QUANTILE_G_q_mean_obs = meanquantiles;
QUANTILE_G_q_median_obs = medianquantiles;

% Estimation of Quantiles of H
DCS_Report_H_Quantiles
QUANTILE_H_p_obs = p_values;
QUANTILE_H_lb_obs = c_lb;
QUANTILE_H_ub_obs = c_ub;
QUANTILE_H_avg_obs = c_avg;

% Storage for variables of interest during bootstrapping
NLLS_p = [];
NLLS_theta_expo = [];
NLLS_q_expo = [];

MC = [];

BOUNDS_min_ub_H = [];
BOUNDS_max_lb_H = [];
BOUNDS_c_H = [];

QUANTILE_H_p = [];
QUANTILE_H_lb = [];
QUANTILE_H_ub = [];

QUANTILE_G_p = [];
QUANTILE_G_q = [];
QUANTILE_G_q_mean = [];
QUANTILE_G_q_median = [];

% Output filename for bootstrap results
if (monotonicity == 1)
    matfile = sprintf('%s_%s_Bootstrap_Results_N=%d_mono.mat', book, method_name, k);
else
    matfile = sprintf('%s_%s_Bootstrap_Results_N=%d_unrs.mat', book, method_name, k);
end

resample = 1;
for r = 1:REP
    disp(sprintf('\nBootstrap Replication %4d\n==========================\n', r));

    % Estimating the period by period G_{t}(c) and mu_{t}'s
    disp(sprintf('\n%s Estimation\n----------------\n', method_name));
    Bootstrapping_Estimation

    % Store marginal costs
    MC(r,:) = R_All;
    MC_mean(r) = mean(R_All);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Estimating the varibles of interest for the different models
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    disp(sprintf('\nBounds Estimation\n-----------------\n'));
    G_c_All = DCS_MU_to_G(NP, k, MU_All); % Convert MGW/W2011 estimates into usable form
    Precalculate_H_Bounds
    BOUNDS_min_ub_H(r,:) = min_ub_H; % Matrix with columns of the outer upper bounds of H(c) and rows corresponding to each rep
    BOUNDS_max_lb_H(r,:) =  max_lb_H; % Matrix with columns of the outer lower bounds of H(c) and rows corresponding to each rep
    BOUNDS_avg_H(r,:) =  avg_H; % Matrix with columns of the average of the bounds for H(c) and rows corresponding to each rep
    BOUNDS_c_H(r,:) = c_H; % Matrix with columns of the cutoff values and rows corresponding to each rep

    % Non-Linear Least Squares Parametric Estimation of H(c|\mu)
    disp(sprintf('\nNLS Estimation\n--------------\n'));
    DCS_Master_NLS

    NLLS_p(r,:)= p_values;
    NLLS_theta_expo(r,:) = theta_expo;
    NLLS_theta_mix(r,:) = theta_mix;

    NLLS_q_expo(r,:) = quantiles_expo;
    NLLS_q_mix(r,:) = quantiles_mix;

    % Estimation of Quantiles of G
    disp(sprintf('\nEstimation of Quantiles of G\n----------------------------\n'));
    DCS_Report_G_Quantiles

    QUANTILE_G_p(r,:) = p_values; % Matrix with the rows corresponding to each rep and the columns are the different p_values
    QUANTILE_G_q(r,:,:) = quantiles'; % 3D matrix with rows corresponding to replications, columns corresponding to quantiles, and depth corresponding to periods 1 to NP
    QUANTILE_G_q_mean(r,:) = meanquantiles;
    QUANTILE_G_q_median(r,:) = medianquantiles;

    % Estimation of Quantiles of H
    disp(sprintf('\nEstimation of Quantiles of H\n----------------------------\n'));
    DCS_Report_H_Quantiles

    QUANTILE_H_p(r,:) = p_values; % Matrix with the rows corresponding to each rep and the columns are the different p_values
    QUANTILE_H_lb(r,:) = c_lb; % Matrix with rows being replications and columns being lower bound quantiles
    QUANTILE_H_ub(r,:) = c_ub; % Matrix with rows being replications and columns being upper bound quantiles
    QUANTILE_H_avg(r,:) = c_avg; % Matrix with rows being replications and columns being average bound quantiles

    % Periodically save intermediate results and save final results
    if ( mod(r, 100) == 0 || r == REP )
        save(matfile, '-regexp', '^NLLS', '^BOUNDS', '^QUANTILE', '^MC');
    end

    % Report intermediate results
    if (r > 1)
        if (r == REP)
            disp(sprintf('\nFinal Results\n-------------\n'));
        else
            disp(sprintf('\nIntermediate Results\n--------------------\n'));
        end

        disp('Exponential distribution (mean parameter):');
        disp([ NLLS_theta_expo_obs, nanstd(NLLS_theta_expo) ]);

        disp('Exponential distribution quantiles:');
        n_quant = size(NLLS_p(r,:));
        valfmt = repmat('%8.2f', [1, n_quant]);
        rowfmt = sprintf('%s%s', '%15s', valfmt);
        disp(sprintf(rowfmt, 'Quantile', NLLS_p(r,:)));
        disp(sprintf(rowfmt, 'Estimate', NLLS_q_expo_obs));
        disp(sprintf(rowfmt, 'S.E.', nanstd(NLLS_q_expo)));

        disp('Exponential mixture (mean 1, mean 2, alpha):');
        disp(sprintf(rowfmt, 'Parameters', NLLS_theta_mix_obs));
        disp(sprintf(rowfmt, 'S.E.', nanstd(NLLS_theta_mix)));

        disp('Exponential mixture quantiles:');
        n_quant = size(NLLS_p(r,:));
        valfmt = repmat('%8.2f', [1, n_quant]);
        rowfmt = sprintf('%s%s', '%15s', valfmt);
        disp(sprintf(rowfmt, 'Quantile', NLLS_p(r,:)));
        disp(sprintf(rowfmt, 'Estimate', NLLS_q_mix_obs));
        disp(sprintf(rowfmt, 'S.E.', nanstd(NLLS_q_mix)));

        disp('Quantiles of lower and upper bounds of H:');
        n_quant = size(QUANTILE_H_p(r,:));
        valfmt = repmat('%8.2f', [1, n_quant]);
        rowfmt = sprintf('%s%s', '%15s', valfmt);
        disp(sprintf(rowfmt, 'Quantile', QUANTILE_H_p(r,:)));
        disp(sprintf(rowfmt, 'Est. Lower', QUANTILE_H_lb_obs));
        disp(sprintf(rowfmt, 'S.E. Lower', nanstd(QUANTILE_H_lb)));
        disp(sprintf(rowfmt, 'Est. Upper', QUANTILE_H_ub_obs));
        disp(sprintf(rowfmt, 'S.E. Upper', nanstd(QUANTILE_H_ub)));
        disp(sprintf(rowfmt, 'Est. Avg.', QUANTILE_H_avg_obs));
        disp(sprintf(rowfmt, 'S.E. Avg.', nanstd(QUANTILE_H_avg)));

        disp('Quantiles of G (selected periods):');
        n_quant = size(QUANTILE_G_p(r,:));
        valfmt = repmat('%8.2f', [1, n_quant]);
        rowfmt = sprintf('%s%s', '%15s', valfmt);
        disp(sprintf(rowfmt, 'Quantile', QUANTILE_G_p(r,:)));
        disp(sprintf(rowfmt, 'Est. t =  25', QUANTILE_G_q_obs(:,25)));
        disp(sprintf(rowfmt, 'S.E. t =  25', nanstd(QUANTILE_G_q(:,:,25))));
        disp(sprintf(rowfmt, 'Est. t =  50', QUANTILE_G_q_obs(:,50)));
        disp(sprintf(rowfmt, 'S.E. t =  50', nanstd(QUANTILE_G_q(:,:,50))));
        disp(sprintf(rowfmt, 'Est. t =  75', QUANTILE_G_q_obs(:,75)));
        disp(sprintf(rowfmt, 'S.E. t =  75', nanstd(QUANTILE_G_q(:,:,75))));
        disp(sprintf(rowfmt, 'Est. t = 100', QUANTILE_G_q_obs(:,100)));
        disp(sprintf(rowfmt, 'S.E. t = 100', nanstd(QUANTILE_G_q(:,:,100))));
        disp(sprintf(rowfmt, 'Est. t = 125', QUANTILE_G_q_obs(:,125)));
        disp(sprintf(rowfmt, 'S.E. t = 125', nanstd(QUANTILE_G_q(:,:,125))));
        disp(sprintf(rowfmt, 'Est. t = 150', QUANTILE_G_q_obs(:,150)));
        disp(sprintf(rowfmt, 'S.E. t = 150', nanstd(QUANTILE_G_q(:,:,150))));
        disp(sprintf(rowfmt, 'Est. t = 175', QUANTILE_G_q_obs(:,175)));
        disp(sprintf(rowfmt, 'S.E. t = 175', nanstd(QUANTILE_G_q(:,:,175))));
        disp(sprintf(rowfmt, 'Est. t = 200', QUANTILE_G_q_obs(:,200)));
        disp(sprintf(rowfmt, 'S.E. t = 200', nanstd(QUANTILE_G_q(:,:,200))));
        disp(sprintf(rowfmt, 'Est. t = 225', QUANTILE_G_q_obs(:,225)));
        disp(sprintf(rowfmt, 'S.E. t = 225', nanstd(QUANTILE_G_q(:,:,225))));
        disp(sprintf(rowfmt, 'Est. Mean', QUANTILE_G_q_mean_obs));
        disp(sprintf(rowfmt, 'S.E. Mean', nanstd(QUANTILE_G_q_mean)));
        disp(sprintf(rowfmt, 'Est. Median', QUANTILE_G_q_median_obs));
        disp(sprintf(rowfmt, 'S.E. Median', nanstd(QUANTILE_G_q_median)));

        disp('Marginal Cost (selected periods):');
        rowfmt = '%15s %8.2f %8.2f';
        disp(sprintf('%15s %8s %8s', 'Period', 'Est.', 'S.E.'));
        disp(sprintf(rowfmt, 'Mean',    MC_mean_obs, nanstd(MC_mean)));
        disp(sprintf(rowfmt, 't =  25', MC_obs(25), nanstd(MC(:,25))));
        disp(sprintf(rowfmt, 't =  50', MC_obs(50), nanstd(MC(:,50))));
        disp(sprintf(rowfmt, 't =  75', MC_obs(75), nanstd(MC(:,75))));
        disp(sprintf(rowfmt, 't = 100', MC_obs(100), nanstd(MC(:,100))));
        disp(sprintf(rowfmt, 't = 125', MC_obs(125), nanstd(MC(:,125))));
        disp(sprintf(rowfmt, 't = 150', MC_obs(150), nanstd(MC(:,150))));
        disp(sprintf(rowfmt, 't = 175', MC_obs(175), nanstd(MC(:,175))));
        disp(sprintf(rowfmt, 't = 200', MC_obs(200), nanstd(MC(:,200))));
        disp(sprintf(rowfmt, 't = 225', MC_obs(225), nanstd(MC(:,225))));
    end
end

% Close worker pool
if (ownpool > 0)
    delete(gcp);
end
