% Jason R. Blevins and Garrett T. Senney
% Dynamic Selection and Distributional Bounds on Search Costs in Dynamic Unit-Demand Models
% Quantitative Economics
%
% Non-linear least squares estimation of H(c | \mu)

% Quantiles to report
p_values = [ 0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99 ];
n_p_values = length(p_values);

% Solver options
options = optimoptions('lsqnonlin','Display','off','MaxFunEvals', 1e10,'FunValCheck','off');
% options = optimoptions(options,'PlotFcns',{@optimplotx @optimplotfunccount @optimplotfval @optimplotstepsize});
nstart = 5;

%% Exponential Distribution
%% ------------------------

% Pregenerate and store random starting values. The Exponential
% distribution has one nonnegative (mean) parameter.
np = 1;
lb = [ 0.0 ];
ub = [ ];
start = zeros(nstart, np);
for j = 1:nstart
    start(j,:) = abs(randn(1, np));
end
if exist('NLLS_theta_expo_obs')
    start(1,:) = NLLS_theta_expo_obs;
end

% Anonymous functions for cdf and objective function
cdf = @(x,theta)expcdf(x,theta(1));
objective = @(theta)DCS_H_NLLS(k, NP, C_All, lb_H, ub_H, cdf, theta);

% NLS estimation for each starting value
theta = zeros(nstart, np);
resnorm = zeros(nstart, 1);
%disp(sprintf('%10s  %10s  %10s  %10s', 'Num', 'Start', 'Estimate', 'Norm'));
for j = 1:nstart
    [ theta(j,:), resnorm(j) ] = lsqnonlin(objective, start(j,:), lb, ub, options);
    %disp(sprintf('%10d  %10.3f  %10.3f  %10.3f', j, start(j,:), theta(j,:), resnorm(j)));
end
[ resnorm_min, j_min ] = min(resnorm);
theta_expo = theta(j_min, 1);

% Report estimate
disp(sprintf('Minimum Q (Expo): %12.5f', resnorm_min));
disp(sprintf('Location of minimum (Expo): %12.5f', theta_expo));

% Calculate quantiles
quantiles_expo = expinv(p_values, theta_expo);

%% Exponential Mixture
%% -------------------

% Pregenerate and store random starting values. Each Exponential
% distribution has one nonnegative (mean) parameter. There is also a
% mixing parameter alpha.
np = 3;
lb = [ 0.0, 0.0, 0.0 ];
ub = [ 100.0, 100.0, 1.0 ];
start = zeros(nstart, np);
for j = 1:nstart
    start(j,1:2) = abs(randn(1, 2));
    start(j,3) = rand(1,1);
end
if exist('NLLS_theta_mix_obs')
    start(1,:) = NLLS_theta_mix_obs;
end

% Anonymous functions for cdf and objective function
cdf = @(x,theta)(theta(3)*expcdf(x,theta(1)) + (1-theta(3))*expcdf(x,theta(1)+theta(2)));
objective = @(theta)DCS_H_NLLS(k, NP, C_All, lb_H, ub_H, cdf, theta);

% NLS estimation for each starting value
theta = zeros(nstart, np);
resnorm = zeros(nstart, 1);
%disp(sprintf('%10s  %10s  %10s  %10s  %10s', 'Num', 'Theta_1', 'Theta_2', 'Alpha', 'Norm'));
for j = 1:nstart
    [ theta(j,:), resnorm(j) ] = lsqnonlin(objective, start(j,:), lb, ub, options);
    %disp(sprintf('%10d  %10.3f  %10.3f  %10.3f  %10.3f', j, theta(j,:), resnorm(j)));
end
[ resnorm_min, j_min ] = min(resnorm);
theta_mix = theta(j_min, :);
theta_mix(2) = theta_mix(1) + theta_mix(2);

% Report estimate
disp(sprintf('Minimum Q (Expo Mixture): %12.5f', resnorm_min));
disp(sprintf('Location of minimum (Expo Mixture): ( %12.5f, %12.5f, %12.5f )', theta_mix));

% Calculate quantiles by solving for zero
zero_options = optimset('TolFun', 1e-6, 'Display', 'off');
quantiles_mix = zeros(size(p_values));
for j = 1:length(p_values)
    quant = @(x)(cdf(x, theta_mix) - p_values(j));
    quantiles_mix(j) = fzero(quant, 1.0, zero_options);
end

%% Report results
%% --------------

% Report quantiles
disp(sprintf('%s (k = %d):', book, k));
valfmt = repmat('%8.2f', 1, size(p_values, 2));
rowfmt = sprintf('%s%s', '%15s', valfmt);
disp(sprintf(rowfmt, 'Quantile', p_values));
disp(sprintf(rowfmt, 'Expo', quantiles_expo));
disp(sprintf(rowfmt, 'ExpoMix', quantiles_mix));

% Variables to save for boostrap:
% theta_expo, theta_mix, quantiles_expo, quantiles_mix, p_values.
