% Jason R. Blevins and Garrett T. Senney
% Dynamic Selection and Distributional Bounds on Search Costs in Dynamic Unit-Demand Models
% Quantitative Economics
%
% Precalculates lower and upper bounds on H for each period and cutoff

lb_H = zeros(NP-1,k);
ub_H = ones(NP-1,k);
c_H = zeros(NP-1,k);

for t = 1:NP-1
    c = C_All(t,:);
    cp = C_All(t+1,:);
    G = G_c_All(t,:);
    Gp = G_c_All(t+1,:);

    for j = 2:k
        % Store c value
        c_H(t,j) = cp(j);

        % Interpolate G and Gp at relevant values, bounded to [ 0, 1 ]
        G_interp = interp1(c, G, cp);
        G_interp = min(max(G_interp, 0), 1);

        % Upper bound: H(c_cur) <= G_{t+1}(c) / G_t(c)
        ub_H(t,j) = min(Gp(j) / G_interp(j), 1);

        % Lower bound with unrestricted sigma:
        % [ G_{t+1}(c) - G_t(c) ] / [ 1 - G_t(c) ] <= H(c)
        lb_H(t,j) = max(Gp(j) - G_interp(j), 0) / (1 - G_interp(j));

        % Lower bound with monotonic sigma: G(c)
        if (monotonicity == 1 & G_interp(j) < Gp(j))
            lb_H(t,j) = max(lb_H(t,j), G_interp(j));
        end
    end
end

% Impose monotonicity of the CDF H
for t = 1:NP-1
    for j = 2:k
        if (j < k)
            ub_H(t,j) = min(ub_H(t,j), ub_H(t,j+1));
        end
        if (j > 1)
            lb_H(t,j) = max(lb_H(t,j), lb_H(t,j-1));
        end
    end
end

% Convert bounds to vector
nc = (NP-1) * k;                        % Number of c values
lb_H_vec = reshape(lb_H, nc, 1);
ub_H_vec = reshape(ub_H, nc, 1);
c_H = reshape(c_H, nc, 1);

% Sorting
[ c_H, sortindex ] = sort(c_H);
lb_H_vec = lb_H_vec(sortindex);
ub_H_vec = ub_H_vec(sortindex);

% Bounding
lb_H_vec = max(lb_H_vec, zeros(nc, 1));
ub_H_vec = min(ub_H_vec, ones(nc, 1));

% Envelope bounds
min_ub_H = ones(nc, 1);
max_lb_H = zeros(nc, 1);
avg_H = zeros(nc, 1);
for j = 1:nc
    % Upper bound
    ind = find(c_H >= c_H(j));
    min_ub_H(j) = min(ub_H_vec(ind));

    % Lower bound
    ind = find(c_H <= c_H(j));
    max_lb_H(j) = max(lb_H_vec(ind));

    % Average bound
    avg_H(j) = (min_ub_H(j) + max_lb_H(j)) / 2;
end
